/**
 * \file snake.cpp
 * \brief Implements the snake instance
 * \see snake.h
 *
 * \date Nov 5, 2010
 * \author Sergey Levi <sergeyle@gmail.com>
 */

#include <ncurses.h>
#include <list>

#include "snake.h"

Snake::Snake(Point start_point, int length, WINDOW* win, char body_symbol) :
    mGameWindow(win),
    mHaveEaten(false),
    mBodySymbol(body_symbol)
{
    for (int i = 0; i < length; i++)
    {
        mSnakeBody.push_back(Point(start_point.row, start_point.col - i));
    }
}

void Snake::moveSnake(Direction dir)
{
    Point &head = mSnakeBody.front();
    Point new_head(0, 0);
    switch (dir)
    {
        case DirectionUp:
        {
            new_head = Point(head.row - 1, head.col);
            break;
        }

        case DirectionRight:
        {
            new_head =  Point(head.row, head.col + 1);
            break;
        }

        case DirectionDown:
        {
            new_head =  Point(head.row + 1, head.col);
            break;
        }

        case DirectionLeft:
        {
            new_head =  Point(head.row, head.col - 1);
            break;
        }
        
    }

    mSnakeBody.push_front(new_head);

    if (!mHaveEaten)
        mSnakeBody.pop_back();
    else
        mHaveEaten = false;
 
}

void Snake::eat()
{
    mHaveEaten = true;
}

void Snake::drawSnake() const
{
    list<Point>::const_iterator it;
    for (it = mSnakeBody.begin(); it != mSnakeBody.end(); it++)
    {
        mvwaddch(mGameWindow, it->row, it->col, mBodySymbol);
    }
    
}

Point Snake::getHead() const
{
    return mSnakeBody.front();
}

bool Snake::hasCollided() const
{
    list<Point>::const_iterator it;
    for (it = ++mSnakeBody.begin(); it != mSnakeBody.end(); it++)
    {
        if (*it == getHead())
            return true;
    }

    return false;
}

