/**
 * \file snake.h
 * \brief Declares the snake instance
 *
 * \date Nov 5, 2010
 * \author Sergey Levi <sergeyle@gmail.com>
 */

#ifndef SNAKE_H_
#define SNAKE_H_

#include <list>
#include <ncurses.h>

using namespace std;

enum Direction
{
    DirectionUp = 0,
    DirectionRight,
    DirectionDown,
    DirectionLeft
};

struct Point
{
public:
    unsigned row;
    unsigned col;

    Point(unsigned _row, unsigned _col) :
        row(_row), col(_col)
    {
    }

    bool operator ==(const Point &other) const
    {
        return ((col == other.col) && (row == other.row));
    }
};

class Snake
{
private:
    list<Point> mSnakeBody;         // List of points consisting the snake's body
    WINDOW*     mGameWindow;        // a handle to the game field window

    bool mHaveEaten;                // set to true if the snake have eaten and should grow
    char mBodySymbol;               // The symbol to be used for the snake's body

public:
    Snake(Point start_point, int length, WINDOW* win, char body_symbol = 'o');

    /**
     * Moves the snake one cell in the given direction
     */

    void moveSnake(Direction dir);

    /**
     * Notifies that the snake has just run over a food cell and should grow
     */

    void eat();

    /**
     * Draws the snake on the game field
     */
    void drawSnake() const;

    /**
     * Returns the coordinates of the snake's head
     */
    Point getHead() const;

    /**
     * Check whether the snake has collided with itself
     */
	bool hasCollided() const;
};

#endif // SNAKE_H_
