/**
 * \file snakeGame.h
 * \brief Declares a game engine for the NCURSES snake game
 *
 * \date Nov 5, 2010
 * \author Sergey Levi <sergeyle@gmail.com>
 */

#ifndef SNAKEGAME_H_
#define SNAKEGAME_H_

#include <ncurses.h>

#include "snake.h"

#define FIELD_TITLE "Score: "

class SnakeGame
{
public:
    
    /**
     * Initializes the game engine
     *
     * \args food_symbol the symbol to be used when drawing food
     */
    SnakeGame(char food_symbol = '*');
    
    /**
     * The main game loop
     */

    void gameLoop();

    ~SnakeGame();

private:
    unsigned    mScore;

    unsigned    mFieldTop;      // Game field position
    unsigned    mFieldLeft;     // and dimensions
    unsigned    mFieldHeight;
    unsigned    mFieldWidth;

    WINDOW*     mPlayWindow;    // handle to game field window
    Snake       mSnake;         // snake instance
    Point       mFoodLocation;  // current location of the food item

    char        mFoodSymbol;    // the symbol to be used for drawing the food

    void drawBoard();
    void printTitle();
    void placeFood();
    void displayMessageModal(const char *msg);
    bool isInside(Point p);
};

#endif // SNAKEGAME_H_
